from peewee import Proxy, SqliteDatabase
from peewee import Model, TextField, IntegerField, ForeignKeyField

database_proxy = Proxy()  # Create a proxy for our db.


class BaseModel(Model):
    class Meta:
        database = database_proxy  # Use proxy for our DB.


class Scan(BaseModel):
    file_hash = TextField(unique=True, null=False)
    name = TextField(null=False)
    type = TextField(null=False)
    start = TextField(null=True)
    end = TextField(null=True)
    elapsed = TextField(null=True)
    hosts_total = IntegerField(null=True)
    hosts_up = IntegerField(null=True)
    hosts_down = IntegerField(null=True)


class Host(BaseModel):
    #hostid = IntegerField(primary_key=True)
    address = TextField(null=False)
    hostname = TextField(null=True)
    os = TextField(null=True)
    os_gen = TextField(null=True)
    status = TextField(null=True)
    scan = ForeignKeyField(Scan, related_name="belongs_to")


class Port(BaseModel):
    #portid = IntegerField(primary_key=True)
    host = ForeignKeyField(Host, related_name="from_host")
    address = TextField(null=False)
    port = IntegerField(null=False)
    protocol = TextField(null=False)
    service = TextField(null=True)
    banner = TextField(null=True)
    status = TextField(null=False)


class Vuln(BaseModel):
    host = ForeignKeyField(Host, related_name="vuln_on_host")
    description = TextField(null=False)
    synopsis = TextField(null=True)
    port = IntegerField(null=False)
    protocol = TextField(null=False)
    service = TextField(null=False)
    solution = TextField(null=True)
    severity = TextField(null=True)
    xref = TextField(null=True)
    info = TextField(null=True)
    plugin_id = TextField(null=False)
    plugin_name = TextField(null=False)
    plugin = TextField(null=True)
    risk = TextField(null=True)


def init_db(db):
    database = SqliteDatabase(db)
    database_proxy.initialize(database)
    database.connect()
    database.create_tables(models=[Scan, Host, Port, Vuln], safe=True)
    return database


import argparse
import sqlite3

HOSTS_BY_CVE = "SELECT distinct address FROM vuln join host on vuln.host_id = host.id WHERE xref like ? and severity >= ?;"
HOSTS_BY_VULN_DESC = "SELECT distinct address FROM vuln join host on vuln.host_id = host.id WHERE description like ? and severity >= ?;"
HOSTS_BY_PID = "SELECT distinct address FROM vuln join host on vuln.host_id = host.id WHERE plugin_id = ? and severity >= ?;"
HOSTS_BY_PNAME = "SELECT distinct address FROM vuln join host on vuln.host_id = host.id WHERE plugin_name like ? and severity >= ?;"

HOSTS_DETAILS_BY_CVE = "SELECT distinct address,port,protocol,severity,plugin_id,plugin_name FROM vuln join host on vuln.host_id = host.id WHERE xref like ? and severity >= ?;"
HOSTS_DETAILS_BY_VULN_DESC = "SELECT distinct address,port,protocol,severity,plugin_id,plugin_name FROM vuln join host on vuln.host_id = host.id WHERE description like ? and severity >= ?;"
HOSTS_DETAILS_BY_PID = "SELECT distinct address,port,protocol,severity,plugin_id,plugin_name FROM vuln join host on vuln.host_id = host.id WHERE plugin_id like ? and severity >= ?;"
HOSTS_DETAILS_BY_PNAME = "SELECT distinct address,port,protocol,severity,plugin_id,plugin_name FROM vuln join host on vuln.host_id = host.id WHERE plugin_name like ? and severity >= ?;"

def get_ips_by_filter(db, query="", search = "", min_severity=0):
    conn = sqlite3.connect(db)
    cur = conn.cursor()
    cur.execute(query, (search,min_severity,))
    rows = cur.fetchall()
    ips = [x[0] for x in rows]
    conn.close()
    return ips


def get_details_by_filter(db, query="", search = "", min_severity=0):
    conn = sqlite3.connect(db)
    cur = conn.cursor()
    cur.execute(query, (search,min_severity,))
    rows = cur.fetchall()
    conn.close()
    return rows

def vulns_cli():
    parser = argparse.ArgumentParser(description="")
    parser.add_argument("--db", type=str, required=False, default="scandb.sqlite")
    parser.add_argument("--min-severity", type=int, required=False, default=0,
                        help="Minimum severity level (default: 0)")
    parser.add_argument("--filter-by", required=False, choices=['cve', 'plugin-id', 'plugin-name', 'description'],
                        default='description', help="Filter hosts by the given filter. The search value is specified with option --search. The following fields can be used as filter 'cve', 'plugin-id', 'plugin-name', 'description")
    parser.add_argument("--search", metavar="SEARCH-Term", required=True, type=str,
                        help="Search term used for querying the database. The type of the search field can be selected "
                             "with the parameter --filter-by")
    parser.add_argument("--list", required=True, choices=['ips', 'details'], default='ips',
                        help="Generate a target list of ip addresses when selecting 'ips' or display the columns"
                             "Address,Port,Protocol,Severity,Plugin-ID,Plugin-Name")
    parser.add_argument("-d", "--list-delimiter", required=False, default="\n",
                        help="Delimiter used to separate hosts in the list output. Only when --list ips is used.")
    parser.add_argument("--list-file", metavar="FILE", required=False, type=str, default=None,
                        help="Generate a file with the targets instead of printing them to stdout")
    args = parser.parse_args()

    if not args.list and args.list_file is None and args.details:
        parser.print_usage()
        return

    ips = []
    details = []

    if args.list == 'ips':
        if args.filter_by == "cve":
            cve = "%{0}%".format(args.search)
            ips = get_ips_by_filter(args.db, HOSTS_BY_CVE, cve, args.min_severity)

        if args.filter_by == "plugin-id":
            ips = get_ips_by_filter(args.db, HOSTS_BY_PID, args.search, args.min_severity)

        if args.filter_by == "plugin-name":
            pn = "%{0}%".format(args.search)
            ips = get_ips_by_filter(args.db, HOSTS_BY_PNAME, pn, args.min_severity)

        if args.filter_by == "description":
            desc = "%{0}%".format(args.search)
            ips = get_ips_by_filter(args.db, HOSTS_BY_VULN_DESC, desc, args.min_severity)

        print(args.list_delimiter.join(ips))

    if args.list == 'details':
        if args.filter_by == "cve":
            cve = "%{0}%".format(args.search)
            details =  get_details_by_filter(args.db, HOSTS_DETAILS_BY_CVE, cve, args.min_severity)
        if args.filter_by == "plugin-id":
            details =  get_details_by_filter(args.db, HOSTS_DETAILS_BY_PID, args.search, args.min_severity)
        if args.filter_by == "plugin-name":
            pn = "%{0}%".format(args.search)
            details =  get_details_by_filter(args.db, HOSTS_DETAILS_BY_PNAME, pn, args.min_severity)
        if args.filter_by == "description":
            desc = "%{0}%".format(args.search)
            details =  get_details_by_filter(args.db, HOSTS_DETAILS_BY_VULN_DESC, desc, args.min_severity)

        fmt = '{0:>20}{1:>15}{2:>15}{3:>15}{4:>15}{5}'
        print(fmt.format("Address","Port","Protocol","Severity","Plugin-ID","Plugin-Name"))
        for i in details:
            addr, port,proto,severity, pid, pn = i
            print(fmt.format(addr, port,proto,severity, pid, pn))
        
    if args.list_file:
        with open(args.list_file, 'w') as f:
            f.write("\n".join(ips))





import argparse
import sqlite3

from scandb.statistics import get_vuln_stats
from scandb.statistics import host_port_list

SQL_ADDR_FROM_HOSTS = "SELECT distinct address from host where status = 'up'"
host_port_list = """
    select address , group_concat(distinct port), protocol from port where protocol = 'tcp' and status='open' group by address
    union
    select address , group_concat(distinct port), protocol from port where protocol = 'udp' and status='open' group by address;"""



def run_query(db, query):
    conn = sqlite3.connect(db)
    cur = conn.cursor()
    cur.execute(query)
    rows = cur.fetchall()
    conn.close()
    return rows


def handle_vuln_stats(db1, db2, outfile="scandb"):

    result = {}
    addresses_db1 = [ x[0] for x in run_query(db1, SQL_ADDR_FROM_HOSTS)]
    addresses_db2 = [ x[0] for x in run_query(db2, SQL_ADDR_FROM_HOSTS)]
    addresses = set(addresses_db1).union(addresses_db2)

    # prepare a dictionary with all ip addresses that exist in one of the databases
    for ip in addresses:
        result[ip] = {'db1': {'critical' : '-', 'high' : '-', 'medium': '-' ,'low': '-', 'info':'-' },
                      'db2' : {'critical' : '-', 'high' : '-', 'medium': '-' ,'low': '-', 'info':'-' }}

    stats_db1 = get_vuln_stats(db1)
    for s in stats_db1:
        ip , c, h, m, l, i = s
        result[ip]['db1']['critical'] = c
        result[ip]['db1']['high'] = h
        result[ip]['db1']['medium'] = m
        result[ip]['db1']['low'] = l
        result[ip]['db1']['info'] = i

    stats_db2 = get_vuln_stats(db2)
    for s in stats_db2:
        ip, c, h, m, l, i = s
        result[ip]['db2']['critical'] = c
        result[ip]['db2']['high'] = h
        result[ip]['db2']['medium'] = m
        result[ip]['db2']['low'] = l
        result[ip]['db2']['info'] = i

    outstr = ['Address;Critical-DB1;Critical-DB2;HIGH-DB1;HIGH-DB2;MEDIUM-DB1;MEDIUM-DB2;LOW-DB1;LOW-DB2;INFO-DB1;INFO-DB2']
    fmt = "{0};{1};{2};{3};{4};{5};{6};{7};{8};{9};{10}"
    for ip in result:
        c1 = result[ip]['db1']['critical']
        h1 = result[ip]['db1']['high']
        m1 = result[ip]['db1']['medium']
        l1 = result[ip]['db1']['low']
        i1 = result[ip]['db1']['info']
        c2 = result[ip]['db2']['critical']
        h2 = result[ip]['db2']['high']
        m2 = result[ip]['db2']['medium']
        l2 = result[ip]['db2']['low']
        i2 = result[ip]['db2']['info']
        outstr.append (fmt.format(ip, c1, h1, m1, l1, i1, c2, h2, m2, l2, i2) )

    filename = "{0}-vulns-db1-db2.csv".format(outfile)
    with open(filename, "w") as f:
        f.write("\n".join(outstr))
    print(" Result written to file: {0}".format(filename))



def handle_service_stats(db1, db2, outfile="scandb"):

    result = {}
    addresses_db1 = [ x[0] for x in run_query(db1, SQL_ADDR_FROM_HOSTS)]
    addresses_db2 = [ x[0] for x in run_query(db2, SQL_ADDR_FROM_HOSTS)]
    addresses = set(addresses_db1).union(addresses_db2)

    # prepare a dictionary with all ip addresses that exist in one of the databases
    for ip in addresses:
        result[ip] = {'db1': {'tcp' : '-', 'udp':'-' },
                      'db2' : {'tcp' : '-', 'udp':'-' }}

    stats_db1 = run_query(db1, host_port_list)
    for i in stats_db1:
        ip , ports, proto = i
        if proto =='tcp':
            result[ip]['db1']['tcp'] = ports
        elif proto == 'udp':
            result[ip]['db1']['udp'] = ports

    stats_db2 = run_query(db2, host_port_list)
    for i in stats_db2:
        ip, ports, proto = i
        if proto == 'tcp':
            result[ip]['db2']['tcp'] = ports
        elif proto == 'udp':
            result[ip]['db2']['udp'] = ports

    outstr = ['Address;tcp-db1;udp-db1;tcp-db2;udp-db2']
    fmt = "{0};{1};{2};{3};{4}"
    for ip in result:
        t1 = result[ip]['db1']['tcp']
        u1 = result[ip]['db1']['udp']
        t2 = result[ip]['db2']['tcp']
        u2 = result[ip]['db2']['udp']
        outstr.append(fmt.format(ip, t1, u1, t2, u2,))

    filename = "{0}-services-db1-db2.csv".format(outfile)
    with open(filename, "w") as f:
        f.write("\n".join(outstr))
    print(" Result written to file: {0}".format(filename))

def compare_cli():
    parser = argparse.ArgumentParser(description="")
    parser.add_argument("--db1", type=str, required=False, default="scandb.sqlite")
    parser.add_argument("--db2", type=str, required=False, default="scandb-old.sqlite")
    parser.add_argument("-v", "--vuln-statistics", required=False, action='store_true', default=False,
                        help="Print number of vulns foreach host and db.")
    #parser.add_argument("-p", "--port-statistics", required=False, action='store_true', default=False,
    #                    help="Print number of 'open' TCP and UDP ports foreach host and db.")
    parser.add_argument("--host-portlist", required=False, action='store_true', default=False,
                        help="generate a csv with a list of TCP and UDP Ports per host and db")
    parser.add_argument("-o", "--outfile", required=False, default="scandb", help="Prefix for output files.")
    args = parser.parse_args()

    if args.vuln_statistics:
        handle_vuln_stats(args.db1, args.db2, args.outfile)

    #if args.port_statistics:
    #    pass

    if args.host_portlist:
        handle_service_stats(args.db1, args.db2, args.outfile)


from __future__ import print_function
import argparse
import os
import peewee
from libnessus.parser import NessusParser
from termcolor import colored
from scandb.util import host_to_tupel, get_ports, hash_file
from scandb.models import Scan, Host, Port, Vuln, init_db

def _nessus_host_to_dbhost(h, scan):
    """
    Create a database host object with the values from the nessus host object.

    :param h: nessus host object
    :type h: libnessus.objects.reporthost.NessusReportHost

    :param scan: database scan object
    :type scan: scandb.models.Scan

    :return: database host object
    :rtype: scandb.models.Host
    """
    host = Host(address=h.ip, hostname=h.name, scan=scan)
    # get non-default values from NessusReportHost object
    netbios = h.get_host_properties.get("netbios-name")
    fqdn = h.get_host_properties.get("host-fqdn")
    operating_system = h.get_host_properties.get("operating-system")
    os = h.get_host_properties.get("os")
    # update the database host if these values where not empty
    if netbios:
        host.hostname = netbios
    if fqdn:
        host.hostname = fqdn
    if os:
        host.os = os
    if operating_system:
        host.os = operating_system
    return host


def _nessus_vuln_to_dbvuln(v, host):
    """

    :param v:
    :type v: libnessus.objects.reportitem.NessusReportItem
    :return:
    """
    vuln = Vuln(host=host, description=v.description, synopsis=v.synopsis, port=v.port, protocol=v.protocol,
                service=v.service, solution=v.solution, severity=v.severity, xref=v.get_vuln_xref, info=v.get_vuln_info,
                plugin=v.get_vuln_plugin, plugin_id=v.plugin_id, plugin_name=v.plugin_name, risk=v.get_vuln_risk)
    return vuln


def import_nessus_file(infile):
    """
    This function is responsable for importing the given file.  For each file a SHA-512 hash is calculated to ensure
    that the file is only imported once.

    :param infile: nessus XML-file to import
    :return:
    """
    print(colored("[*] Importing file: {0}".format(infile), 'green'))
    try:
        report = NessusParser.parse_fromfile(infile)
        # calculate a SHA-512 hash. This is used to ensure that the file will not be imported more than once.
        sha512 = hash_file(infile)

        # create the database entry for the scan.
        scan = Scan(file_hash=sha512, name=report.name, type='nessus', start=report.started, end=report.endtime,
                    elapsed=report.elapsed, hosts_total=report.hosts_total)
        scan.save()

        # import all hosts and ports present in the report
        for h in report.hosts:
            host = _nessus_host_to_dbhost(h, scan=scan)
            host.save()
            for v in h.get_report_items:
                vuln = _nessus_vuln_to_dbvuln(v, host)
                vuln.save()
        print(colored("[*] File imported. ", 'green'))
    except peewee.IntegrityError as e:
        # This error is throw when the SHA-512 hash is already present in the database. Therefore the file cannot be
        # imported again.
        print(colored("[-] File already imported: {0}".format(infile), 'red'))
        print(colored("[-] {0}".format(e.message), 'red'))
    except Exception as e:
        # Invalid file format
        print(colored("[-] {0}".format(e.message), 'red'))
